package cn.fw.valhalla.component;

import cn.fw.member.sdk.event.MemberMergeEvent;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.service.data.CustomerBaseInfoService;
import cn.fw.valhalla.service.data.CustomerService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 会员合并mq消费
 *
 * @author : kurisu
 * @version : 1.0
 * @className : MemberRemoveConsumer
 * @description : 会员移除mq消费
 * @date : 2022-07-04 09:55
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = MemberMergeEvent.TOPIC, consumerGroup = "${spring.application.name}" + "-" + MemberMergeEvent.TOPIC)
public class MemberMergeConsumer implements RocketMQListener<MemberMergeEvent> {


    private final CustomerService customerService;
    private final CustomerBaseInfoService customerBaseInfoService;

    @Autowired
    public MemberMergeConsumer(final CustomerService customerService,
                               final CustomerBaseInfoService customerBaseInfoService) {
        this.customerService = customerService;
        this.customerBaseInfoService = customerBaseInfoService;
    }


    @Override
    public void onMessage(MemberMergeEvent t) {
        final String str = "会员合并mq";
        log.info("{}: MemberMergeEvent=[{}]", str, t);
        try {
            Long memberId = t.getMemberId();
            List<Long> invalidList = t.getInvalidList();
            mergeCustomer(memberId, invalidList);
        } catch (Exception ex) {
            log.error("{}失败，原因:{}", str, JSON.toJSONString(ex));
            throw ex;
        }
    }

    /**
     * 合并档案
     *
     * @param memberId
     * @param invalidList
     */
    public void mergeCustomer(final Long memberId, final List<Long> invalidList) {
        if (CollectionUtils.isEmpty(invalidList)) {
            return;
        }
        List<CustomerBaseInfo> list = customerBaseInfoService.list(Wrappers.<CustomerBaseInfo>lambdaQuery()
                .in(CustomerBaseInfo::getMemberId, invalidList)
                .eq(CustomerBaseInfo::getYn, Boolean.TRUE)
        );

        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        Map<Long, Long> groupBaseInfo = new HashMap<>();

        for (CustomerBaseInfo baseInfo : list) {
            List<Customer> customerList = customerService.queryByBaseId(baseInfo.getId());
            baseInfo.setYn(Boolean.FALSE);
            if (CollectionUtils.isEmpty(customerList)) {
                continue;
            }
            if (!Objects.equals(memberId, baseInfo.getMemberId())) {
                CustomerBaseInfo realBaseInfo = customerBaseInfoService.queryByMemberId(memberId, baseInfo.getGroupId());
                if (Objects.isNull(realBaseInfo)) {
                    if (Objects.isNull(groupBaseInfo.putIfAbsent(baseInfo.getGroupId(), baseInfo.getId()))) {
                        baseInfo.setMemberId(memberId);
                        baseInfo.setYn(Boolean.TRUE);
                    }
                } else {
                    groupBaseInfo.putIfAbsent(baseInfo.getGroupId(), realBaseInfo.getId());
                }
                for (Customer customer : customerList) {
                    customer.setBaseId(groupBaseInfo.get(customer.getGroupId()));
                }
                customerService.updateBatchById(customerList);
            }
        }
        customerBaseInfoService.updateBatchById(list);
    }
}
